/*
 * protein_query.h
 *
 *  Created on: 2011/04/13
 *      Author: shu
 */

#ifndef NORMAL_QUERY_H_
#define NORMAL_QUERY_H_

#include "query.h"
#include "encoder.h"
#include "statistics.h"
#include "aligner_parameters.h"
#include <vector>

using namespace std;

class ProteinQuery : public Query{
public:
  static uint32_t GetSequenceLength(uint32_t length) {
    if (length == 0) {
      return 0;
    } else {
      return length + 2;
    }
  }
  ProteinQuery(const Sequence &sequence, AlignerParameters &parameters);

  virtual uint32_t GetRealSequenceLength() {
    return sequence_.size() - 2;
  }
  
  SequenceIndex *GetIndex() {
    if (index_ == NULL) {
      SetIndex(sequence_, encoder_);
    }
    return index_;
  }

  KarlinParameters GetUngappedKarlinParameters() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    } 
    return ungapped_karlin_parameters_;
  }

  int GetUngappedExtendNominalCutoff() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    } 
    return ungapped_extend_nominal_cutoff_;
  }

  int GetGappedExtendNominalCutoff() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    } 
    return gapped_extend_nominal_cutoff_;
  }

  int GetGappedExtendNominalTrigger() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    } 
    return gapped_extend_nominal_trigger_;
  }


private:
  static const uint32_t kSegWindow = 12;
  void SetName(const Sequence &sequence);
  void SetEncodedSequence(const Sequence &sequence, const Encoder &encoder, const bool filter);
  void SetIndex(const vector<Code> &sequence, const Encoder &encoder);
  void SetStaticalParameters(const vector<Code> &sequence, const AlignerParameters &parameters);
  AlignerParameters *parameters_;
  Encoder encoder_;
  bool is_set_statical_parameters_;
};

#endif /* NORMAL_QUERY_H_ */
